#!/usr/bin/python
# coding=utf-8
# Python Version 2.7
# motor.py
# Schrittmotorsteuerung mit Kamerabildanzeige im Browser über Tornado Websocket
#------------------------------------------------------------------------------

# Bibliotheken
#--------------------
# Einbindung der notwendigen Grundbibliotheken
import os, sys, time, threading

# Einbindung GPIO's
import RPi.GPIO as gp

# Einbindung der notwendigen Bibliotheken für den Tornado Websocket
import tornado.httpserver, tornado.websocket, tornado.ioloop, tornado.web, datetime, json

# Einbindung der Datenbank - in diesem Fall SQLite3
import sqlite3

# Einbindung der Kamera
import picamera

# Deklarationen
#--------------------
# Global für Programmstatus
programmStatus = 1 
# Datenbank mit der gespeicherten Stellung der Schrittmotorstellung
sqlitemotorStellungDBName = "/var/www/html/sqlite/motorstellung.db" 


# Global für Schrittmotor
schrittMotorStellung = 0 # Stellung des Schrittmotors

# Global für Videostream
dateiNameStream = '' # zur Übergabe des aktuellen Dateinamen an die Weboberfläche
Kamera_Stream_aktiv = False # als Erweiterung der folgenden Streaming-Option per Socket
kameraStreamStatus = False # notwendig für die Kamerasteuerung
bilderStreamPfad = "/var/www/html/streamtmp/" # Ram-Disk

# Websocket
#----------
class TornadoThread (threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        application = tornado.web.Application([(r'/ws', WSHandler),])
        http_server = tornado.httpserver.HTTPServer(application)
        http_server.listen(8888)

    def run(self):
        print "Tornado Websocketdienst gestartet.\nWarte auf Verbindung.\n...\n"
        tornado.ioloop.IOLoop.instance().start()

class WSHandler(tornado.websocket.WebSocketHandler):
    def check_origin(self, origin):
        return True
    
    def open(self):
        global alterDatenUebergabeWert
        alterDatenUebergabeWert = ''
        self.connected = True
        print "Verbindung mit WEB-Oberfläche aufgebaut.\n"
        self.timeout_loop()
 
    def on_message(self, message):
        global Kamera_Stream_aktiv

        print "Übergabewert von WEB-Interface: ", message

        if message == "page_2":
            print "User befindet sich auf motor.html - setze Kamera_Stream_aktiv auf True"
            Kamera_Stream_aktiv = True
            message = ""
        elif message == "cam-left":
            print "Schrittmotor in linke Richtung drehen lassen"
            schrittmotor("positiv")
        elif message == "cam-right":
            print "Schrittmotor in rechte Richtung drehen lassen"
            schrittmotor("negativ")
        else:
            print "unbekannter Befehl"
                    
    def on_close(self):
        self.connected = False
        print "Verbindung zum WEB-Interface geschlossen."
        self.timeout_loop()

    def timeout_loop(self):
        global Kamera_Stream_aktiv, dateiNameStream
 
        if self.connected:
            if Kamera_Stream_aktiv == True:
                self.write_message(dateiNameStream)
            tornado.ioloop.IOLoop.instance().add_timeout(datetime.timedelta(seconds=.1), self.timeout_loop)

# Funktionen
#-----------------------------

def gpioInitalisieren():
    # GPIO zur Verwendung vorbereiten
    try:
        gp.setmode(gp.BCM) # BCM Nummern verwenden
        gp.setwarnings(False) # Warnmeldungen abschalten
        # GPIO Ports für das ULN2003A Treiberboard für den 28BYJ-48 Schrittmotor initalisieren
        gp.setup(20, gp.OUT) # Pin38 - BCM Nummer 20 - GPIO20 wird Ausgang = IN1
        gp.output(20, gp.LOW)
        gp.setup(16, gp.OUT) # Pin36 - BCM Nummer 16 - GPIO16 wird Ausgang = IN2
        gp.output(16, gp.LOW)
        gp.setup(19, gp.OUT) # Pin35 - BCM Nummer 19 - GPIO19 wird Ausgang = IN3
        gp.output(19, gp.LOW)
        gp.setup(13, gp.OUT) # Pin33 - BCM Nummer 13 - GPIO13 wird Ausgang = IN4
        gp.output(13, gp.LOW)
    except:
        # Fehler bei der GPIO-Initialisierung
        programmStatus =  102

def schrittmotor(richtung):
    global schrittMotorStellung
    # damit die aktuelle Position nach einem Neustart bekannt ist, wird die Position abgespeichert
    # damit es evtl. keine Probleme mit den Kabelverbindungen gibt, wird die Bewegung eingschränkt
    bewegungsStop = 130
    print "aktuelle schrittMotorStellung = ", schrittMotorStellung
    if richtung == "negativ":
        print "Negativ"
        i = 1
        zukuenftigeSchrittMotorStellung = schrittMotorStellung - 16
        if zukuenftigeSchrittMotorStellung > -bewegungsStop:
            while i < 16 :
                schrittmotorStepNegativ()
                i = i + 1
            schrittMotorStellung = schrittMotorStellung - i
        else:
            print "keine Bewegung da außerhalb gesetzter Position"
    elif richtung == "positiv":
        print "Positiv"
        i = 1
        zukuenftigeSchrittMotorStellung = schrittMotorStellung + 16
        if zukuenftigeSchrittMotorStellung < bewegungsStop:
            while i < 16 :
                schrittmotorStepPositiv()
                i = i + 1
            schrittMotorStellung = schrittMotorStellung + i
        else:
            print "keine Bewegung da außerhalb gesetzter Position"
    sql = 'UPDATE schrittmotorstellung SET position = "' + str(schrittMotorStellung) + '"'
    motorStellung_db_update(sql)

def schrittmotorStepPositiv():
    zeit = 0.01
    #Step 1
    gp.output(13, gp.HIGH)
    time.sleep(zeit)
    gp.output(13, gp.LOW)
    #Step 2
    gp.output(13, gp.HIGH)
    gp.output(19, gp.HIGH)
    time.sleep(zeit)
    gp.output(13, gp.LOW)
    gp.output(19, gp.LOW)
    #Step 3
    gp.output(19, gp.HIGH)
    time.sleep(zeit)
    gp.output(19, gp.LOW)
    #Step 4
    gp.output(16, gp.HIGH)
    gp.output(19, gp.HIGH)
    time.sleep(zeit)
    gp.output(16, gp.LOW)
    gp.output(19, gp.LOW)
    #Step 5
    gp.output(16, gp.HIGH)
    time.sleep(zeit)
    gp.output(16, gp.LOW)
    #Step 6
    gp.output(20, gp.HIGH)
    gp.output(16, gp.HIGH)
    time.sleep(zeit)
    gp.output(20, gp.LOW)
    gp.output(16, gp.LOW)
    #Step 7
    gp.output(20, gp.HIGH)
    time.sleep(zeit)
    gp.output(20, gp.LOW)
    #Step 8
    gp.output(13, gp.HIGH)
    gp.output(20, gp.HIGH)
    time.sleep(zeit)
    gp.output(13, gp.LOW)
    gp.output(20, gp.LOW)

def schrittmotorStepNegativ():
    zeit = 0.01
    #Step 8
    gp.output(13, gp.HIGH)
    gp.output(20, gp.HIGH)
    time.sleep(zeit)
    gp.output(13, gp.LOW)
    gp.output(20, gp.LOW)
    #Step 7
    gp.output(20, gp.HIGH)
    time.sleep(zeit)
    gp.output(20, gp.LOW)
    #Step 6
    gp.output(20, gp.HIGH)
    gp.output(16, gp.HIGH)
    time.sleep(zeit)
    gp.output(20, gp.LOW)
    gp.output(16, gp.LOW)
    #Step 5
    gp.output(16, gp.HIGH)
    time.sleep(zeit)
    gp.output(16, gp.LOW)
    #Step 4
    gp.output(16, gp.HIGH)
    gp.output(19, gp.HIGH)
    time.sleep(zeit)
    gp.output(16, gp.LOW)
    gp.output(19, gp.LOW)
    #Step 3
    gp.output(19, gp.HIGH)
    time.sleep(zeit)
    gp.output(19, gp.LOW)
    #Step 2
    gp.output(13, gp.HIGH)
    gp.output(19, gp.HIGH)
    time.sleep(zeit)
    gp.output(13, gp.LOW)
    gp.output(19, gp.LOW)
    #Step 1
    gp.output(13, gp.HIGH)
    time.sleep(zeit)
    gp.output(13, gp.LOW)


def motorStellung_db_anlegen():
    global sqlitemotorStellungDBName, schrittMotorStellung
    connection = sqlite3.connect(sqlitemotorStellungDBName)
    cursor = connection.cursor()
    sql = "CREATE TABLE schrittmotorstellung(" \
        "position INTEGER)"
    cursor.execute(sql)
    sql = "INSERT INTO schrittmotorstellung VALUES('"+ str(schrittMotorStellung) + "')"
    cursor.execute(sql)
    connection.commit()
    connection.close()

def motorStellung_db_auslesen():
    global sqlitemotorStellungDBName, schrittMotorStellung
    dbSensorBezeichnung = []
    dbSensorName = []
    connection = sqlite3.connect(sqlitemotorStellungDBName)
    cursor = connection.cursor()
    sql = "SELECT * FROM schrittmotorstellung"
    cursor.execute(sql)
    for dsatz in cursor:
        print "ausgelesene Daten für Motorstellung ", dsatz[0]
        schrittMotorStellung = dsatz[0]
    connection.close()

def motorStellung_db_update(sql):
    global sqlitemotorStellungDBName
    connection = sqlite3.connect(sqlitemotorStellungDBName)
    cursor = connection.cursor()
    cursor.execute(sql)
    print "Datenbank für Motorstellung aktualisiert mit ", sql
    connection.commit()    
    connection.close()

#GPIO initialisieren
gpioInitalisieren()

# Überprüfung Existenz der SQLite3-Datei für Schrittmotorstellung
if (not os.path.isfile(sqlitemotorStellungDBName)):
    motorStellung_db_anlegen()
else:
    motorStellung_db_auslesen()  

# Festlegung Tornado Websocket
TornadoThread = TornadoThread()
TornadoThread.start()
print "Programminformation: es sind Insgesamt ", threading.active_count(), " Threads aktiv."

# Hauptroutine
i = 0
while programmStatus == 1:
    if Kamera_Stream_aktiv == True:
        if kameraStreamStatus == False:
            # Kamera initialisieren
            cam = picamera.PiCamera()
            cam.resolution = (640, 480) # Auflösung für Bildspeicherung
            print "Bildaufnahme für Stream beginn"
            kameraStreamStatus = True
        cam.start_preview()
        dateiName = bilderStreamPfad + str(i) + ".jpg"
        time.sleep(0.05)
        cam.capture(dateiName)
        dateiNameStream = "stream|streamtmp/" + str(i) + ".jpg"
        cam.stop_preview()
        i = i + 1
        if i == 100:
            i = 0
    else:
        if kameraStreamStatus == True:
            cam.close() # damit die Kamera nicht ständig in Betrieb ist
            kameraStreamStatus = False
            i = 0   
               
# Programmende durch Veränderung des programmStatus
# GPIO Einstellungen löschen
gp.cleanup()
# Ausgabe der Nachricht für das Programmende
print "Programm wurde wegen eines Fehlers beendet. Fehlercode: ", programmStatus
if programmStatus == 102:
    print "Die Initialisierung des GPIO ist fehlgeschlagen."
else:
    print "Der Fehlercode wurde nicht hinterlegt."